@@ -42,7 +42,7 @@
 #include "driver.h"
 #include "dri2.h"
 
-#ifdef GLAMOR
+#ifdef GLAMOR_HAS_GBM
 
 enum ms_dri2_frame_event_type {
     MS_DRI2_QUEUE_SWAP,
@@ -511,11 +511,14 @@ update_front(DrawablePtr draw, DRI2BufferPtr front)
     ms_dri2_buffer_private_ptr priv = front->driverPrivate;
     CARD32 size;
     CARD16 pitch;
+    int name;
 
-    front->name = glamor_name_from_pixmap(pixmap, &pitch, &size);
-    if (front->name < 0)
+    name = glamor_name_from_pixmap(pixmap, &pitch, &size);
+    if (name < 0)
         return FALSE;
 
+    front->name = name;
+
     (*screen->DestroyPixmap) (priv->pixmap);
     front->pitch = pixmap->devKind;
     front->cpp = pixmap->drawable.bitsPerPixel / 8;
@@ -1081,4 +1084,4 @@ ms_dri2_close_screen(ScreenPtr screen)
     DRI2CloseScreen(screen);
 }
 
-#endif /* GLAMOR */
+#endif /* GLAMOR_HAS_GBM */
